/*	7_19.cpp
	This program simulates the computer Simpletron.
	It asks for a file, and reads the commands from that file.
	If the command is out of acceptable range, it is discarded.
	The memory contents are displayed after the execution of the
	program.
*/
#include <stdio.h>

/* Command Definitions */
#define READ		10
#define WRITE		11
#define LOAD		20
#define STORE		21
#define ADD			30
#define SUBTRACT	31
#define DIVIDE		32
#define MULTIPLY	33
#define BRANCH		40
#define BRANCHNEG	41
#define BRANCHZERO	42
#define HALT		43

/* Array size definitions */
#define STRING_SIZE 11
#define MEMORY_SIZE 100

int main()
{
	FILE *commandFilePtr;
	char strFileName[STRING_SIZE];
	int memoryArray[MEMORY_SIZE] = { 0 };
	int numberOfCommands = 0;
	int i;

	int errorFlag = 0;

	int accumulator = 0,
		instructionCounter = 0, 
		operationCode = 0,
		operand = 0,
		instructionRegister = 0;

	printf( "Enter the command file name ->" );
	scanf( "%s", strFileName );

	if ( ( commandFilePtr = fopen( strFileName, "r" ) ) == NULL ) {
		printf( "*** File cannot be opened! ***\n" );
		return 0;
	}
	
	/* Read the commands from the file */
	while ( !feof( commandFilePtr ) ) {
		fscanf( commandFilePtr, "%d", &memoryArray[ numberOfCommands ] );
		
		if ( (memoryArray[ numberOfCommands ] >= -9999) && (memoryArray[ numberOfCommands ] <= 9999 ) )
			numberOfCommands++; /* Discard unidentified commands! */
	}
	printf( "*** %d commands succesfully read! ***\n", numberOfCommands );
	
	fclose( commandFilePtr );

	/* Start execution of program */
	printf( "*** Program execution starts! ***\n" );
	while ( operationCode != HALT ) {
		
		/* Read instruction from memory */
		instructionRegister = memoryArray[ instructionCounter ]; 

		operationCode = instructionRegister / 100; /* Get left two digits */
		operand		  = instructionRegister % 100; /* Get right two digits */

		switch (operationCode) {
		
		/* Read input from keyboard */
		case READ:
			if ( operand <= MEMORY_SIZE - 1 ) {
				printf( "Keyboard entry : " );
				scanf( "%d", &memoryArray[ operand ] );
				instructionCounter++; /* Increase instructionCounter */
			} else
				errorFlag = 1;
			break;

		/* Write output to the screen */
		case WRITE:
			if ( operand <= MEMORY_SIZE - 1 ) {
				printf( "Program output : %d\n", memoryArray[ operand ] );
				instructionCounter++; /* Increase instructionCounter */
			} else
				errorFlag = 1;
			break;

		/* Load value from memory to accumulator */ 
		case LOAD:
			if ( operand <= MEMORY_SIZE - 1 ) {
				accumulator = memoryArray[ operand ];
				instructionCounter++; /* Increase instructionCounter */
			} else
				errorFlag = 1;
			break;
		
		/* Store accumulator value to a memory location */ 
		case STORE:
			if ( operand <= MEMORY_SIZE - 1 ) {
				memoryArray[ operand ] = accumulator;
				instructionCounter++; /* Increase instructionCounter */
			} else
				errorFlag = 1;
			break;

		/* Add memory content to the accumulator */
		case ADD:
			if ( operand <= MEMORY_SIZE - 1 ) {
				accumulator += memoryArray[ operand ];
				instructionCounter++; /* Increase instructionCounter */
			} else
				errorFlag = 1;
			break;

		/* Subtract memory content from accumulator */
		case SUBTRACT:
			if ( operand <= MEMORY_SIZE - 1 ) {
				accumulator -= memoryArray[ operand ];
				instructionCounter++; /* Increase instructionCounter */
			} else
				errorFlag = 1;
			break;

		/* Divide the accumulator to the memory content */
		case DIVIDE:
			if ( memoryArray[ operand ] == 0 ) {/* Divide by zero error. */
				printf( "*** Divide by zero error! ***\n" );
				printf( "*** Program execution abnormally aborted!\n" );
				errorFlag = 1;
			}
			if ( operand <= MEMORY_SIZE - 1 ) {
				accumulator /= memoryArray[ operand ];
				instructionCounter++; /* Increase instructionCounter */
			} else
				errorFlag = 1;
			break;
		
		/* Multiply the memory content with accumulator */
		case MULTIPLY:
			if ( operand <= MEMORY_SIZE - 1 ) {
				accumulator *= memoryArray[ operand ];
				instructionCounter++; /* Increase instructionCounter */
			} else
				errorFlag = 1;
			break;

		/* Unconditional branch */
		case BRANCH:
			if ( operand <= MEMORY_SIZE - 1 ) {
				instructionCounter = operand;
			} else
				errorFlag = 1;
			break;

		/* Branch if accumulator content is negative */
		case BRANCHNEG:
			if ( operand <= MEMORY_SIZE - 1 ) {
				if ( accumulator < 0 )
					instructionCounter = operand;
				else
					instructionCounter++; /* Increase instructionCounter */
			} else
				errorFlag = 1;
			break;

		/* Branch if accumulator content is zero */
		case BRANCHZERO:
			if ( operand <= MEMORY_SIZE - 1 ) {
				if ( accumulator == 0 )
					instructionCounter = operand;
				else
					instructionCounter++; /* Increase instructionCounter */
			} else
				errorFlag = 1;
			break;

		/* Halt the program execution */
		case HALT:
			printf( "Program execution halted!\n" );
			break;

		default:
			errorFlag = 1;
		}

		/* Check the error flag */
		if ( errorFlag != 0 ) {
			printf( "Illegal operation '%d' at memory location : %d.\n", instructionRegister, instructionCounter);
			break;
		}

		/* Check the accumulator content for overflows */
		if ( (accumulator > 9999) || (accumulator < -9999)) {
			printf( "*** accumulator overflow! ***\n" );
			printf( "*** Program execution abnormally aborted! ***\n");
			break;
		}

	}

	/* Print out the memory contents */
	printf( "\nMemory\n" );
	printf( "accumulator : %.4d\n", accumulator); 
	printf(	"instructionCounter : %.4d\n", instructionCounter );
	printf(	"instructionRegister : %.4d\n", instructionRegister );
	printf(	"operationCode : %.4d\n", operationCode );
	printf(	"operand : %.4d\n\n", operand );
	
	printf( "%9.2d%7.2d%7.2d%7.2d%7.2d%7.2d%7.2d%7.2d%7.2d%7.2d", 
			0, 1, 2, 3, 4, 5, 6, 7, 8, 9);
	for ( i = 0; i < MEMORY_SIZE; i++ ) {
		if ( (i%10) == 0) {
			printf( "\n%.2d %7.4d", i, memoryArray[i] );
		} else {
			printf( "%7.4d", memoryArray[i] );
		}
	}
	printf( "\n" );
	
	return 0;
}